@@ -36,7 +36,7 @@
 
 #include "sd-daemon.h"
 #include "build.h"
-#include "def.h"
+#include "constants.h"
 #include "sd-event.h"
 #include "log.h"
 #include "list.h"
@@ -46,9 +46,9 @@
 #include "fd-util.h"
 #include "string-util.h"
 #include "io-util.h"
-#include "util.h"
 #include "alloc-util.h"
 #include "locale-util.h"
+#include "logarithm.h"
 
 #define FSCKD_SOCKET_PATH "/run/systemd/fsck.progress"
 #define IDLE_TIME_SECONDS 30
@@ -109,6 +109,11 @@ static Manager* manager_free(Manager *m);
 DEFINE_TRIVIAL_CLEANUP_FUNC(Client*, client_free);
 DEFINE_TRIVIAL_CLEANUP_FUNC(Manager*, manager_free);
 
+static void init_gettext(void) {
+        setlocale(LC_ALL, "");
+        textdomain(GETTEXT_PACKAGE);
+}
+
 static bool plymouth_running(void) {
         return access("/run/plymouth/pid", F_OK) >= 0;
 }
@@ -210,7 +215,6 @@ static void manager_disconnect_plymouth(Manager *m) {
 
 static int manager_plymouth_feedback_handler(sd_event_source *s, int fd, uint32_t revents, void *userdata) {
         Manager *m = userdata;
-        Client *current;
         char buffer[6];
         ssize_t l;
 
@@ -243,7 +247,10 @@ static int manager_plymouth_feedback_handler(sd_event_source *s, int fd, uint32_
 }
 
 static int manager_connect_plymouth(Manager *m) {
-        union sockaddr_union sa = PLYMOUTH_SOCKET;
+        union sockaddr_union sa = {
+                .un.sun_family = AF_UNIX,
+                .un.sun_path = "\0/org/freedesktop/plymouthd",
+        };
         int r;
 
         if (!plymouth_running())
@@ -286,7 +293,7 @@ static int plymouth_send_message(int plymouth_fd, const char *message, bool upda
         if (asprintf(&packet, "%c\002%c%s%n", mode, (int) (strlen(message) + 1), message, &n) < 0)
                 return log_oom();
 
-        return loop_write(plymouth_fd, packet, n + 1, true);
+        return loop_write_full(plymouth_fd, packet, n + 1, USEC_INFINITY);
 }
 
 static int manager_send_plymouth_message(Manager *m, const char *message) {
@@ -303,7 +310,7 @@ static int manager_send_plymouth_message(Manager *m, const char *message) {
         if (!m->plymouth_cancel_sent) {
 
                 /* Indicate to plymouth that we listen to Ctrl+C */
-                r = loop_write(m->plymouth_fd, PLYMOUTH_REQUEST_KEY, sizeof(PLYMOUTH_REQUEST_KEY), true);
+                r = loop_write_full(m->plymouth_fd, PLYMOUTH_REQUEST_KEY, sizeof(PLYMOUTH_REQUEST_KEY), USEC_INFINITY);
                 if (r < 0)
                         return log_warning_errno(r, "Can't send to plymouth cancel key: %m");
 
@@ -333,7 +340,6 @@ static int manager_send_plymouth_message(Manager *m, const char *message) {
 }
 
 static int manager_update_global_progress(Manager *m) {
-        Client *current = NULL;
         _cleanup_free_ char *console_message = NULL;
         _cleanup_free_ char *fsck_message = NULL;
         int current_numdevices = 0, r;
@@ -625,7 +631,7 @@ static int parse_argv(int argc, char *argv[]) {
         assert(argc >= 0);
         assert(argv);
 
-        while ((c = getopt_long(argc, argv, "hv", options, NULL)) >= 0)
+        while ((c = getopt_long(argc, argv, "h", options, NULL)) >= 0)
                 switch (c) {
 
                 case 'h':
@@ -640,7 +646,7 @@ static int parse_argv(int argc, char *argv[]) {
                         return -EINVAL;
 
                 default:
-                        assert_not_reached("Unhandled option");
+                        assert_not_reached();
                 }
 
         if (optind < argc) {
