@@ -18,9 +18,10 @@
 #include "bus-log-control-api.h"
 #include "bus-map-properties.h"
 #include "bus-polkit.h"
+#include "bus-unit-util.h"
 #include "clock-util.h"
 #include "conf-files.h"
-#include "def.h"
+#include "constants.h"
 #include "fd-util.h"
 #include "fileio-label.h"
 #include "fileio.h"
@@ -31,6 +32,7 @@
 #include "memory-util.h"
 #include "missing_capability.h"
 #include "path-util.h"
+#include "ro-etc-hack.h"
 #include "selinux-util.h"
 #include "service-util.h"
 #include "signal-util.h"
@@ -102,18 +104,19 @@ static void unit_status_info_clear(UnitStatusInfo *p) {
         p->active_state = mfree(p->active_state);
 }
 
-static void unit_status_info_free(UnitStatusInfo *p) {
-        assert(p);
+static UnitStatusInfo *unit_status_info_free(UnitStatusInfo *p) {
+        if (!p)
+                return NULL;
 
         unit_status_info_clear(p);
         free(p->name);
         free(p->path);
-        free(p);
+        return mfree(p);
 }
 
-static void context_clear(Context *c) {
-        UnitStatusInfo *p;
+DEFINE_TRIVIAL_CLEANUP_FUNC(UnitStatusInfo*, unit_status_info_free);
 
+static void context_clear(Context *c) {
         assert(c);
 
         free(c->zone);
@@ -122,14 +125,15 @@ static void context_clear(Context *c) {
 
         sd_bus_slot_unref(c->slot_job_removed);
 
-        while ((p = c->units)) {
-                LIST_REMOVE(units, c->units, p);
-                unit_status_info_free(p);
-        }
+        LIST_CLEAR(units, c->units, unit_status_info_free);
 }
 
 static int context_add_ntp_service(Context *c, const char *s, const char *source) {
-        UnitStatusInfo *u;
+        _cleanup_(unit_status_info_freep) UnitStatusInfo *unit = NULL;
+
+        assert(c);
+        assert(s);
+        assert(source);
 
         if (!unit_name_is_valid(s, UNIT_NAME_PLAIN))
                 return -EINVAL;
@@ -139,18 +143,17 @@ static int context_add_ntp_service(Context *c, const char *s, const char *source
                 if (streq(u->name, s))
                         return 0;
 
-        u = new0(UnitStatusInfo, 1);
-        if (!u)
+        unit = new0(UnitStatusInfo, 1);
+        if (!unit)
                 return -ENOMEM;
 
-        u->name = strdup(s);
-        if (!u->name) {
-                free(u);
+        unit->name = strdup(s);
+        if (!unit->name)
                 return -ENOMEM;
-        }
 
-        LIST_APPEND(units, c->units, u);
-        log_unit_debug(u, "added from %s.", source);
+        LIST_APPEND(units, c->units, unit);
+        log_unit_debug(unit, "added from %s.", source);
+        TAKE_PTR(unit);
 
         return 0;
 }
@@ -190,7 +193,6 @@ static int context_parse_ntp_services_from_environment(Context *c) {
 
 static int context_parse_ntp_services_from_disk(Context *c) {
         _cleanup_strv_free_ char **files = NULL;
-        char **f;
         int r;
 
         r = conf_files_list_strv(&files, ".list", NULL, CONF_FILES_FILTER_MASKED, UNIT_LIST_DIRS);
@@ -210,9 +212,8 @@ static int context_parse_ntp_services_from_disk(Context *c) {
 
                 for (;;) {
                         _cleanup_free_ char *line = NULL;
-                        const char *word;
 
-                        r = read_line(file, LINE_MAX, &line);
+                        r = read_stripped_line(file, LINE_MAX, &line);
                         if (r < 0) {
                                 log_error_errno(r, "Failed to read %s, ignoring: %m", *f);
                                 continue;
@@ -220,13 +221,12 @@ static int context_parse_ntp_services_from_disk(Context *c) {
                         if (r == 0)
                                 break;
 
-                        word = strstrip(line);
-                        if (isempty(word) || startswith(word, "#"))
+                        if (isempty(line) || startswith(line, "#"))
                                 continue;
 
-                        r = context_add_ntp_service(c, word, *f);
+                        r = context_add_ntp_service(c, line, *f);
                         if (r < 0)
-                                log_warning_errno(r, "Failed to add NTP service \"%s\", ignoring: %m", word);
+                                log_warning_errno(r, "Failed to add NTP service \"%s\", ignoring: %m", line);
                 }
         }
 
@@ -244,7 +244,6 @@ static int context_parse_ntp_services(Context *c) {
 }
 
 static int context_ntp_service_is_active(Context *c) {
-        UnitStatusInfo *info;
         int count = 0;
 
         assert(c);
@@ -258,7 +257,6 @@ static int context_ntp_service_is_active(Context *c) {
 }
 
 static int context_ntp_service_exists(Context *c) {
-        UnitStatusInfo *info;
         int count = 0;
 
         assert(c);
@@ -290,30 +288,9 @@ static int context_read_data(Context *c) {
         return 0;
 }
 
-/* Hack for Ubuntu phone: check if path is an existing symlink to
- * /etc/writable; if it is, update that instead */
-static const char* writable_filename(const char *path) {
-        ssize_t r;
-        static char realfile_buf[PATH_MAX];
-        _cleanup_free_ char *realfile = NULL;
-        const char *result = path;
-        int orig_errno = errno;
-
-        r = readlink_and_make_absolute(path, &realfile);
-        if (r >= 0 && startswith(realfile, "/etc/writable")) {
-                snprintf(realfile_buf, sizeof(realfile_buf), "%s", realfile);
-                result = realfile_buf;
-        }
-
-        errno = orig_errno;
-        return result;
-}
-
 static int context_write_data_timezone(Context *c) {
         _cleanup_free_ char *p = NULL;
         const char *source;
-        int r = 0;
-        struct stat st;
 
         assert(c);
 
@@ -327,12 +304,9 @@ static int context_write_data_timezone(Context *c) {
                 if (access("/usr/share/zoneinfo/UTC", F_OK) < 0) {
 
                         if (unlink(writable_filename("/etc/localtime")) < 0 && errno != ENOENT)
-                                r = -errno;
+                                return -errno;
 
-                        if (unlink(writable_filename("/etc/timezone")) < 0 && errno != ENOENT)
-                                r = -errno;
-
-                        return r;
+                        return 0;
                 }
 
                 source = "/usr/share/zoneinfo/UTC";
@@ -344,17 +318,7 @@ static int context_write_data_timezone(Context *c) {
                 source = p;
         }
 
-        r = symlink_atomic(source, writable_filename("/etc/localtime"));
-        if (r < 0)
-                return r;
-
-        if (stat(writable_filename("/etc/timezone"), &st) == 0 && S_ISREG(st.st_mode)) {
-                r = write_string_file(writable_filename("/etc/timezone"), c->zone, WRITE_STRING_FILE_CREATE|WRITE_STRING_FILE_ATOMIC);
-                if (r < 0)
-                        return r;
-        }
-
-        return 0;
+        return symlink_atomic(source, writable_filename("/etc/localtime"));
 }
 
 static int context_write_data_local_rtc(Context *c) {
@@ -423,7 +387,7 @@ static int context_write_data_local_rtc(Context *c) {
                 }
         }
 
-        r = mac_selinux_init();
+        r = mac_init();
         if (r < 0)
                 return r;
 
@@ -437,7 +401,6 @@ static int context_update_ntp_status(Context *c, sd_bus *bus, sd_bus_message *m)
                 { "UnitFileState", "s", NULL, offsetof(UnitStatusInfo, unit_file_state) },
                 {}
         };
-        UnitStatusInfo *u;
         int r;
 
         assert(c);
@@ -479,13 +442,11 @@ static int context_update_ntp_status(Context *c, sd_bus *bus, sd_bus_message *m)
 }
 
 static int match_job_removed(sd_bus_message *m, void *userdata, sd_bus_error *error) {
-        Context *c = userdata;
-        UnitStatusInfo *u;
+        Context *c = ASSERT_PTR(userdata);
         const char *path;
         unsigned n = 0;
         int r;
 
-        assert(c);
         assert(m);
 
         r = sd_bus_message_read(m, "uoss", NULL, &path, NULL, NULL);
@@ -584,7 +545,7 @@ static int unit_enable_or_disable(UnitStatusInfo *u, sd_bus *bus, sd_bus_error *
         if (r < 0)
                 return r;
 
-        r = bus_call_method(bus, bus_systemd_mgr, "Reload", error, NULL, NULL);
+        r = bus_service_manager_reload(bus);
         if (r < 0)
                 return r;
 
@@ -641,10 +602,9 @@ static int property_get_can_ntp(
                 void *userdata,
                 sd_bus_error *error) {
 
-        Context *c = userdata;
+        Context *c = ASSERT_PTR(userdata);
         int r;
 
-        assert(c);
         assert(bus);
         assert(property);
         assert(reply);
@@ -670,10 +630,9 @@ static int property_get_ntp(
                 void *userdata,
                 sd_bus_error *error) {
 
-        Context *c = userdata;
+        Context *c = ASSERT_PTR(userdata);
         int r;
 
-        assert(c);
         assert(bus);
         assert(property);
         assert(reply);
@@ -691,12 +650,11 @@ static int property_get_ntp(
 }
 
 static int method_set_timezone(sd_bus_message *m, void *userdata, sd_bus_error *error) {
-        Context *c = userdata;
+        Context *c = ASSERT_PTR(userdata);
         int interactive, r;
         const char *z;
 
         assert(m);
-        assert(c);
 
         r = sd_bus_message_read(m, "sb", &z, &interactive);
         if (r < 0)
@@ -770,12 +728,11 @@ static int method_set_timezone(sd_bus_message *m, void *userdata, sd_bus_error *
 
 static int method_set_local_rtc(sd_bus_message *m, void *userdata, sd_bus_error *error) {
         int lrtc, fix_system, interactive;
-        Context *c = userdata;
+        Context *c = ASSERT_PTR(userdata);
         struct timespec ts;
         int r;
 
         assert(m);
-        assert(c);
 
         r = sd_bus_message_read(m, "bbb", &lrtc, &fix_system, &interactive);
         if (r < 0)
@@ -859,14 +816,13 @@ static int method_set_time(sd_bus_message *m, void *userdata, sd_bus_error *erro
         sd_bus *bus = sd_bus_message_get_bus(m);
         char buf[FORMAT_TIMESTAMP_MAX];
         int relative, interactive, r;
-        Context *c = userdata;
+        Context *c = ASSERT_PTR(userdata);
         int64_t utc;
         struct timespec ts;
         usec_t start;
         struct tm tm;
 
         assert(m);
-        assert(c);
 
         if (c->slot_job_removed)
                 return sd_bus_error_set(error, BUS_ERROR_AUTOMATIC_TIME_SYNC_ENABLED, "Previous request is not finished, refusing.");
@@ -951,14 +907,12 @@ static int method_set_time(sd_bus_message *m, void *userdata, sd_bus_error *erro
 static int method_set_ntp(sd_bus_message *m, void *userdata, sd_bus_error *error) {
         _cleanup_(sd_bus_slot_unrefp) sd_bus_slot *slot = NULL;
         sd_bus *bus = sd_bus_message_get_bus(m);
-        Context *c = userdata;
-        UnitStatusInfo *u;
+        Context *c = ASSERT_PTR(userdata);
         const UnitStatusInfo *selected = NULL;
         int enable, interactive, q, r;
 
         assert(m);
         assert(bus);
-        assert(c);
 
         r = sd_bus_message_read(m, "bb", &enable, &interactive);
         if (r < 0)
@@ -1083,42 +1037,31 @@ static const sd_bus_vtable timedate_vtable[] = {
         SD_BUS_PROPERTY("TimeUSec", "t", property_get_time, 0, 0),
         SD_BUS_PROPERTY("RTCTimeUSec", "t", property_get_rtc_time, 0, 0),
 
-        SD_BUS_METHOD_WITH_NAMES("SetTime",
-                                 "xbb",
-                                 SD_BUS_PARAM(usec_utc)
-                                 SD_BUS_PARAM(relative)
-                                 SD_BUS_PARAM(interactive),
-                                 NULL,,
-                                 method_set_time,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("SetTimezone",
-                                 "sb",
-                                 SD_BUS_PARAM(timezone)
-                                 SD_BUS_PARAM(interactive),
-                                 NULL,,
-                                 method_set_timezone,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("SetLocalRTC",
-                                 "bbb",
-                                 SD_BUS_PARAM(local_rtc)
-                                 SD_BUS_PARAM(fix_system)
-                                 SD_BUS_PARAM(interactive),
-                                 NULL,,
-                                 method_set_local_rtc,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("SetNTP",
-                                 "bb",
-                                 SD_BUS_PARAM(use_ntp)
-                                 SD_BUS_PARAM(interactive),
-                                 NULL,,
-                                 method_set_ntp,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("ListTimezones",
-                                 NULL,,
-                                 "as",
-                                 SD_BUS_PARAM(timezones),
-                                 method_list_timezones,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("SetTime",
+                                SD_BUS_ARGS("x", usec_utc, "b", relative, "b", interactive),
+                                SD_BUS_NO_RESULT,
+                                method_set_time,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("SetTimezone",
+                                SD_BUS_ARGS("s", timezone, "b", interactive),
+                                SD_BUS_NO_RESULT,
+                                method_set_timezone,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("SetLocalRTC",
+                                SD_BUS_ARGS("b", local_rtc, "b", fix_system, "b", interactive),
+                                SD_BUS_NO_RESULT,
+                                method_set_local_rtc,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("SetNTP",
+                                SD_BUS_ARGS("b", use_ntp, "b", interactive),
+                                SD_BUS_NO_RESULT,
+                                method_set_ntp,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("ListTimezones",
+                                SD_BUS_NO_ARGS,
+                                SD_BUS_RESULT("as", timezones),
+                                method_list_timezones,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
 
         SD_BUS_VTABLE_END,
 };
